#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "IL/il.h"
#include "IL/ilu.h"

/* Convert an image to the specified format */
int main(int argc, char *argv[])
{
  #define MAX_FILE_LEN   256
  #define MAX_ZOOM_TYPE  9

  int zoomTypeLkup[MAX_ZOOM_TYPE] =
  {
    ILU_NEAREST,         /* 0 */
    ILU_LINEAR,          /* 1 */
    ILU_BILINEAR,        /* 2 */
    ILU_SCALE_BOX,       /* 3 */
    ILU_SCALE_TRIANGLE,  /* 4 */
    ILU_SCALE_BELL,      /* 5 */
    ILU_SCALE_BSPLINE,   /* 6 */
    ILU_SCALE_LANCZOS3,  /* 7 */
    ILU_SCALE_MITCHELL,  /* 8 */
  };

  ILuint imageName;
  char sourceFile[MAX_FILE_LEN] = "";
  char destFile[MAX_FILE_LEN] = "";
  ILuint width, height;
  ILuint zWidth, zHeight;
  double zoom = -1;
  int userZoomType = 7;
  int zoomType = ILU_SCALE_LANCZOS3;

  /* Check for valid number of inputs */
  if(argc < 3)
  {
    printf("ConvertImageFormat by Chris Brochtrup\n");
    printf("Usage: ConvertImageFormat.exe <source image> <destination image> [zoom factor] [zoom type]\n\n");
    exit(1);
  }

  /* Get the source image and dest image */
  strcpy_s(sourceFile, MAX_FILE_LEN, argv[1]);
  strcpy_s(destFile, MAX_FILE_LEN, argv[2]);

  /* Get the option zoom factor */
  if(argc >= 4)
  {
    zoom = atof(argv[3]);
  }

  if(zoom > 0)
  {
    /* Get the option zoom factor */
    if(argc >= 5)
    {
      userZoomType = atoi(argv[4]);

      if(userZoomType < 0 || userZoomType >= MAX_ZOOM_TYPE)
      {
        userZoomType = 7;
      }
    }
  }

  zoomType = zoomTypeLkup[userZoomType];

  #ifdef _DEBUG
    printf("Source File      = %s\n", sourceFile);
    printf("Destination File = %s\n", destFile);
    printf("zoom             = %f\n", zoom);
    printf("userZoomType     = %d\n", userZoomType);
    printf("zoomType         = 0x%X\n", zoomType);
  #endif /* _DEBUG */

  /* Init DevIL */
  ilInit();
  iluInit();

  /* Load the source image */
  ilGenImages(1, &imageName);
  ilBindImage(imageName);
  ilLoadImage(sourceFile);

  if(zoom > 0)
  {
    width = ilGetInteger(IL_IMAGE_WIDTH);
    height = ilGetInteger(IL_IMAGE_HEIGHT);

    #ifdef _DEBUG
      printf("Orig             = %d x %d\n", width, height);
    #endif /* _DEBUG */

    zWidth = (int)((double)width * zoom);
    zHeight = (int)((double)height * zoom);

    #ifdef _DEBUG
      printf("Zoomed           = %d x %d\n", zWidth, zHeight);
    #endif /* _DEBUG */

    iluImageParameter(ILU_FILTER, zoomType);
    iluScale(zWidth, zHeight, 24);
  }

  /* Overwrite image if it already exists */
  ilEnable(IL_FILE_OVERWRITE);

  /* Save the image. DevIL will use extension to determine format. */
  ilSaveImage(destFile);

  ilDeleteImages(1, &imageName);

  return 0;

} /* main */
